#version 330

uniform sampler2D MainSampler;
uniform sampler2D DataSampler;
uniform sampler2D BlurSampler;

uniform vec2 OutSize;
uniform vec2 ScreenSize;
uniform float GameTime;

#moj_import <shader_selector:marker_settings.glsl>
#moj_import <shader_selector:utils.glsl>
#moj_import <shader_selector:data_reader.glsl>

in vec2 texCoord;

out vec4 fragColor;

const float granularity = 40.0;
const mat2 m = mat2( 0.80,  0.60, -0.60,  0.80 );

// NOISE SHIT

float noise( in vec2 p )
{
    return sin(p.x)*sin(p.y);
}

float fbm4( vec2 p )
{
    float f = 0.0;
    f += 0.5000*noise( p ); p = m*p*2.02;
    f += 0.2500*noise( p ); p = m*p*2.02;
    f += 0.1250*noise( p ); p = m*p*2.02;
    f += 0.0625*noise( p );
    return f/0.9375;
}

float fbm6( vec2 p )
{
    float f = 0.0;
    f += 0.500000*(0.5+0.5*noise( p )); p = m*p*2.02;
    f += 0.500000*(0.5+0.5*noise( p )); p = m*p*2.02;
    f += 0.500000*(0.5+0.5*noise( p )); p = m*p*2.02;
    f += 0.250000*(0.5+0.5*noise( p )); p = m*p*2.02;
    return f/0.96875;
}

vec2 fbm4_2( vec2 p )
{
    return vec2(fbm4(p), fbm4(p+vec2(7.8)));
}

vec2 fbm6_2( vec2 p )
{
    return vec2(fbm6(p+vec2(16.8)), fbm6(p+vec2(11.5)));
}

float fbm( vec2 q, out vec4 ron )
{
    float realTime = GameTime * 3600.;

    q += 0.03*sin( vec2(0.27,0.23)*realTime + length(q)*vec2(4.1,4.3));

    vec2 o = fbm4_2( 0.9*q );

    o += 0.04*sin( vec2(0.12,0.14)*realTime + length(o));

    vec2 n = fbm6_2( 3.0*o );

    ron = vec4( o, n );

    float f = 0.5 + 0.5*fbm4( 1.8*q + 6.0*n );

    return mix( f, f*f*f*3.5, f*abs(n.x) );
}

// MAIN

void main() {
    fragColor = texture(MainSampler, texCoord);
    vec2 fragCoord = texCoord * OutSize;
    vec2 uv = fragCoord.xy/ScreenSize;

    // Apply custom darker inversion with minimal green effect
    float wildmuttAmount = readChannel(WILDMUTT_CHANNEL);

    // Custom Inversion with a very subtle green and mostly black
    vec3 customInversion = vec3(0.0, (1.0 - fragColor.g) * 0.3, 0.0);

    // Blend original and modified colors
    vec4 outColor = mix(fragColor, vec4(customInversion, fragColor.a), wildmuttAmount);
    fragColor = vec4(outColor.rgb, 1.0);


    // Transformation effect
    float transformAmount = readChannel(TRANSFORM_CHANNEL);
    float detransformAmount = readChannel(DETRANSFORM_CHANNEL);
    
    vec2 p = (granularity*4.0*floor(gl_FragCoord.xy/granularity)-ScreenSize.xy)/ScreenSize.y;
    float transformCenter = smoothstep(0.3, 0.6/transformAmount, distance(uv, vec2(0.5)));
    float detransformCenter = smoothstep(0.2, 0.8/detransformAmount, distance(uv, vec2(0.5)));

    vec4 on = vec4(0.0);
    float transformNoiseA = fbm(p*0.5+vec2(0.0, ScreenSize.y*GameTime*0.05), on);
    float transformNoiseB = fbm(p*0.55+vec2(0.0, ScreenSize.y*GameTime*-0.2), on);

    fragColor = mix(
        fragColor, 
        ((vec4(pow(vec3(transformNoiseA), vec3(2.0)), transformNoiseA))*vec4(0.3,1.0,0.0,1.0)  *  (vec4(pow(vec3(transformNoiseB), vec3(2.0)), transformNoiseB))*vec4(1.0,1.0,0.0,1.0)), 
        (transformCenter * (transformNoiseA+transformNoiseB))
    );

    fragColor = mix(
        fragColor, 
        ((vec4(pow(vec3(transformNoiseA), vec3(2.0)), transformNoiseA))*vec4(1.0,0.0,0.0,1.0)  +  (vec4(pow(vec3(transformNoiseB), vec3(2.0)), transformNoiseB))*vec4(1.0,0.0,0.2,1.0)), 
        (detransformCenter * (transformNoiseA+transformNoiseB))
    );

    // Detransformation pulsing effect
    float pulseAmount = readChannel(PULSE_CHANNEL);
    float pulseCenter = smoothstep(0.3, 0.8, distance(uv, vec2(0.5)));

    fragColor += vec4(pulseCenter * (0.5 + 0.5 * -cos(pulseAmount * 3.14159265359 * 10)), 0.0, 0.0, 0.0);


#ifdef DEBUG
    // Show data sampler on screen
    if (texCoord.x < .25 && texCoord.y < .25) {
        uv = texCoord * 4.0;
        vec4 col = texture(DataSampler, uv);
        if (uv.x > 1./5. )
            col = vec4(vec3(fract(decodeColor(col))), 1.0);
        fragColor = col;
    }
#endif
}
